@@ -1,6 +1,6 @@
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 // Mobius Forensic Toolkit
-// Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019 Eduardo Aguiar
+// Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019,2020 Eduardo Aguiar
 //
 // This program is free software; you can redistribute it and/or modify it
 // under the terms of the GNU General Public License as published by the
@@ -19,9 +19,9 @@
 #include <mobius/io/file.h>
 #include <mobius/datetime/timedelta.h>
 #include <mobius/string_functions.h>
-#include <mobius/regex_impl.h>
 #include <mobius/exception.inc>
 #include <mobius/charset.h>
+#include <regex>
 #include <stdexcept>
 
 namespace mobius
@@ -48,7 +48,7 @@ imagefile_impl_dossier::is_instance (const std::string& url)
 {
   bool instance = false;
 
-  mobius::io::file f (url);
+  auto f = mobius::io::new_file_by_url (url);
 
   if (f && f.exists ())
     {
@@ -197,55 +197,56 @@ imagefile_impl_dossier::_load_metadata () const
     return;
 
   // load metadata
-  mobius::io::file f (url_);
+  auto f = mobius::io::new_file_by_url (url_);
   constexpr mobius::io::file::size_type LOG_MAX_SIZE = 65536;
 
   if (f.exists () && f.get_size () < LOG_MAX_SIZE)
     {
-      mobius::regex REGEX_DOSSIER_SERIAL ("Serial No.:([0-9]+)");
-      mobius::regex REGEX_DOSSIER_SOFTWARE ("Software: ([A-Z0-9.]+)");
-      mobius::regex REGEX_TIME_ZONE ("Time Zone: ([0-9+-]+)");
-      mobius::regex REGEX_DRIVE_MODEL ("Model : ([^|]+?)\\|");
-      mobius::regex REGEX_DRIVE_SERIAL ("Serial: ([0-9A-Z-]+) *\\|");
-      mobius::regex REGEX_TOTAL_SECTORS ("Total Sectors.*\r\n\\* *([0-9]+).*\\|");
-      mobius::regex REGEX_COMPLETION_TIME ("Completion Time: ([0-9]{2})/([0-9]{2})/([0-9]{4}) *([0-9]{2}):([0-9]{2}):([0-9]{2})");
+      std::regex REGEX_DOSSIER_SERIAL ("Serial No.:([0-9]+)");
+      std::regex REGEX_DOSSIER_SOFTWARE ("Software: ([A-Z0-9.]+)");
+      std::regex REGEX_TIME_ZONE ("Time Zone: ([0-9+-]+)");
+      std::regex REGEX_DRIVE_MODEL ("Model : ([^|]+?)\\|");
+      std::regex REGEX_DRIVE_SERIAL ("Serial: ([0-9A-Z-]+) *\\|");
+      std::regex REGEX_TOTAL_SECTORS ("Total Sectors.*\r\n\\* *([0-9]+).*\\|");
+      std::regex REGEX_COMPLETION_TIME ("Completion Time: ([0-9]{2})/([0-9]{2})/([0-9]{4}) *([0-9]{2}):([0-9]{2}):([0-9]{2})");
 
       // parse .log file
       auto reader = f.new_reader ();
       mobius::bytearray data = reader.read (f.get_size ());
       const std::string text = conv_charset_to_utf8 (data, "ASCII");
+      std::smatch match;
 
       int timezone = 0;
 
-      if (REGEX_TIME_ZONE.match (text))
-        timezone = stoi (REGEX_TIME_ZONE[1]);
+      if (std::regex_search (text, match, REGEX_TIME_ZONE))
+        timezone = stoi (match[1].str ());
 
-      if (REGEX_DOSSIER_SERIAL.match (text))
-        acquisition_platform_ = "Logicube Forensic Dossier (s/n: " + REGEX_DOSSIER_SERIAL[1] + ')';
+      if (std::regex_search (text, match, REGEX_DOSSIER_SERIAL))
+        acquisition_platform_ = "Logicube Forensic Dossier (s/n: " + match[1].str () + ')';
 
-      if (REGEX_DOSSIER_SOFTWARE.match (text))
-        acquisition_tool_ = "Dossier software " + REGEX_DOSSIER_SOFTWARE[1];
+      if (std::regex_search (text, match, REGEX_DOSSIER_SOFTWARE))
+        acquisition_tool_ = "Dossier software " + match[1].str ();
 
-      if (REGEX_DRIVE_MODEL.match (text))
-        drive_model_ = mobius::string::strip (REGEX_DRIVE_MODEL[1]);
+      if (std::regex_search (text, match, REGEX_DRIVE_MODEL))
+        drive_model_ = mobius::string::strip (match[1].str ());
 
-      if (REGEX_DRIVE_SERIAL.match (text))
-        drive_serial_number_ = REGEX_DRIVE_SERIAL[1];
+      if (std::regex_search (text, match, REGEX_DRIVE_SERIAL))
+        drive_serial_number_ = match[1].str ();
 
-      if (REGEX_TOTAL_SECTORS.match (text))
+      if (std::regex_search (text, match, REGEX_TOTAL_SECTORS))
         {
-          sectors_ = stoll (REGEX_TOTAL_SECTORS[1]);
+          sectors_ = stoll (match[1].str ());
           size_ = sectors_ * sector_size_;
         }
 
-      if (REGEX_COMPLETION_TIME.match (text))
+      if (std::regex_search (text, match, REGEX_COMPLETION_TIME))
         {
-          mobius::datetime::datetime d (stoi (REGEX_COMPLETION_TIME[3]),
-                                        stoi (REGEX_COMPLETION_TIME[1]),
-                                        stoi (REGEX_COMPLETION_TIME[2]),
-                                        stoi (REGEX_COMPLETION_TIME[4]),
-                                        stoi (REGEX_COMPLETION_TIME[5]),
-                                        stoi (REGEX_COMPLETION_TIME[6]));
+          mobius::datetime::datetime d (stoi (match[3].str ()),
+                                        stoi (match[1].str ()),
+                                        stoi (match[2].str ()),
+                                        stoi (match[4].str ()),
+                                        stoi (match[5].str ()),
+                                        stoi (match[6].str ()));
           if (timezone)
             {
               mobius::datetime::timedelta delta;
