@@ -1,6 +1,6 @@
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 // Mobius Forensic Toolkit
-// Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018 Eduardo Aguiar
+// Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019,2020 Eduardo Aguiar
 //
 // This program is free software; you can redistribute it and/or modify it
 // under the terms of the GNU General Public License as published by the
@@ -20,8 +20,19 @@
 //! \brief C++ API module wrapper
 //! \author Eduardo Aguiar
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+#include <pymobius.h>
 #include "reader.h"
-#include "api.h"
+
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief Check if object type is <i>reader</i>
+//! \param pyobj Python object
+//! \return true/false
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+bool
+PyMobius_IO_Reader_Check (PyObject *pyobj)
+{
+  return PyObject_IsInstance (pyobj, (PyObject *) &io_reader_t);
+}
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 //! \brief create new object from C++ object
@@ -49,6 +60,17 @@ PyMobius_IO_Reader_from_cpp (mobius::io::reader obj)
   return ret;
 }
 
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief Create <i>reader</i> C++ object from Python object
+//! \param pyobj Python object
+//! \return reader object
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+mobius::io::reader
+PyMobius_IO_Reader_as_cpp (PyObject *pyobj)
+{
+  return * (reinterpret_cast <io_reader_o *>(pyobj)->obj);
+}
+
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 //! \brief is_seekable getter
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -59,7 +81,7 @@ tp_getter_is_seekable (io_reader_o *self)
 
   try
     {
-      ret = PyBool_FromLong (self->obj->is_seekable ());
+      ret = mobius::py::pybool_from_bool (self->obj->is_seekable ());
     }
   catch (const std::exception& e)
     {
@@ -79,7 +101,7 @@ tp_getter_is_rewindable (io_reader_o *self)
 
   try
     {
-      ret = PyBool_FromLong (self->obj->is_rewindable ());
+      ret = mobius::py::pybool_from_bool (self->obj->is_rewindable ());
     }
   catch (const std::exception& e)
     {
@@ -99,7 +121,7 @@ tp_getter_size (io_reader_o *self)
 
   try
     {
-      ret = PyInt_FromLong (self->obj->get_size ());
+      ret = PyLong_FromLong (self->obj->get_size ());
     }
   catch (const std::exception& e)
     {
@@ -136,29 +158,43 @@ static PyGetSetDef tp_getset[] =
 };
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief read method
+//! \brief Read method
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 static PyObject *
 tp_f_read (io_reader_o *self, PyObject *args)
 {
-  PyObject *ret = nullptr;
+  std::uint64_t arg_size = 0;
 
+  // parse input args
   try
     {
-      // parse input args
-      std::uint64_t arg_size = self->obj->get_size () - self->obj->tell ();
+      arg_size = self->obj->get_size () - self->obj->tell ();
 
       if (!PyArg_ParseTuple (args, "|K", &arg_size))
         return nullptr;
-
-      // build value
-      ret = PyString_from_bytearray (self->obj->read (arg_size));
     }
   catch (const std::exception& e)
     {
       PyErr_SetString (PyExc_IOError, e.what ());
     }
 
+  // execute C++ code
+  PyObject *ret = nullptr;
+  PyThreadState *_save = PyEval_SaveThread ();
+
+  try
+    {
+      auto data = self->obj->read (arg_size);
+
+      PyEval_RestoreThread (_save);
+      ret = mobius::py::pybytes_from_bytearray (data);
+    }
+  catch (const std::runtime_error& e)
+    {
+      PyEval_RestoreThread(_save);
+      PyErr_SetString (PyExc_IOError, e.what ());
+    }
+
   return ret;
 }
 
@@ -172,7 +208,7 @@ tp_f_tell (io_reader_o *self, PyObject *)
 
   try
     {
-      ret = PyInt_FromLong (self->obj->tell ());
+      ret = PyLong_FromLong (self->obj->tell ());
     }
   catch (const std::exception& e)
     {
@@ -208,7 +244,7 @@ tp_f_seek (io_reader_o *self, PyObject *args)
 
   else
     {
-      PyErr_SetString (PyExc_ValueError, "invalid 'whence' argument");
+      PyErr_SetString (PyExc_ValueError, "Invalid 'whence' argument");
       return nullptr;
     }
 
